/**
 * texter.js
 *
 * author: Johannes Fröhlich
 */
(function (window, undefined) {
	"use strict";

	function Texter(content) {
		// Everything except null or undefined should be parsed as string.
		if (typeof content === 'undefined' || typeof content === 'null') {
			content = "";
		}
		this.text = String(content);
	}

	var fn = Texter.fn = Texter.prototype;
	
	fn.lower = function () {
		this.text.toLowerCase();
		return this;
	};

	fn.upper = function () {
		this.text.toUpperCase();
		return this;
	};

	/**
	 * Converts the initial letter of every word to uppercase and all remaining
	 * letters to lowercase.
	 */
	fn.capitalize = function () {
		// TODO Implement this method.
		return this;
	};

	/**
	 * Converts the content text into real title case.
	 */
	fn.title = function () {
		// TODO Implement this method.
		return this;
	};
	
	/**
	 * Will convert the text to camel case text.
	 *
	 * Example: "This is my String" will get "ThisIsMyString".
	 */
	fn.camelCase = function () {
		// TODO Implement this method.
		return this;
	};

	/**
	 * Capitalize the first character of the content text.
	 *
	 * Example. "my name" will get "My name"
	 */
	fn.capFirst = function () {
		// TODO Implement this method.
		return this;
	};

	/**
	 * Trim whitespace at the beginning or the end of the text.
	 * 
	 * Uses the ES5 trim method of the String object when available.
	 */
	fn.trim = function () {
		var trim = String.prototype.trim || function () {
			return String(this).replace(/^\s+|\s+$/g, '');
		};
		this.text = trim.call(this.text);
		return this;
	};

	/**
	 * Will remove all occurances of the given value from the content string.
	 *
	 * Example: " " will replace all spaces from "My name" to "Myname"
	 *
	 * @param {string} value
	 *		A string 
	 */
	fn.strip = function () {
		// TODO Implement this method.
		return this;
	};

	/**
	 * Will reverse the content text.
	 */
	fn.reverse = function () {
		var i = this.text.length - 1,
			str = '',
			t = this.text;
		while (i >= 0) {
			str = str + t[i--];
		}
		this.text = str;
		return this;
	};

	/**
	 * Renders variables to placeholders in the content string.
	 *
	 * @param {mixed} ctx 
	 *		An array or object with values that should replace the placeholders.
	 *
	 * @return {Texter}
	 *		The modified texter object.
	 */
	fn.render = function (context) {
		context = (typeof context === 'object') ? context : Array.prototype.slice.call(arguments, 1);
		this.content = this.content.replace(/\{\{|\}\}|\{(\w+)\}/g, function (m, n) {
			if (m === '{{') {
				return '{';
			} else if (m === '}}') {
				return '}';
			}
			return context[n];
		});
		return this;
	};

	/**
	 *	Escapes the content text for use in an URL.
	 *
	 *	Example: "http://www.example.com/foo?a=b&c=d" will 
	 *	get "http%3A//www.example.com/foo%3Fa%3Db%26c%3Dd"
	 */
	fn.urlEncode = function () {
		// TODO Implement this method.
		return this;
	};

	/**
	 * Appends a value to the content text.
	 *
	 * @param {mixed} value
	 *		The value to be appended to the content text.
	 */
	fn.append = function () {
		// TODO Implement this method.
		return this;
	};

	/**
	 * Puts the value at the beginning of the content text.
	 *
	 * @param {mixed} value
	 *		The value to be prepended to the content text.
	 */
	fn.prepend = function () {
		// TODO Implement this method.
		return this;
	};

	/**
	 * Converts the content string into a clean URL name.
	 *
	 * Converts the content text into lowercase, removes non-word characters 
	 * and converts spaces to the given separator.
	 */
	fn.slugify = function () {
		// TODO Implement this method.
		return this;
	};

	/**
	 * Truncate the content text to a fixed number of characters.
	 *
	 * @param {number} value
	 *		The number of characters the content text should be truncated.
	 */
	fn.truncateChars = function () {
		// TODO Implement this method.
		return this;
	};

	/**
	 * Truncate the content text to a fixed number of words.
	 *
	 * @param {number} value
	 *		The number of words the content text should be truncated.
	 *
	 * @param {string} symbol
	 *		A string that should be placed at the end if the text was truncated.
	 */
	fn.truncateWords = function () {
		// TODO Implement this method.
		return this;
	};

	/* H */
	
	/**
	 * Converts numbers from 1-10 to spelled numbers.
	 */
	fn.spellNumbers = function () {
		// TODO Implement this method.
		return this;
	};

	/**
	 * Format all numbers in the content text to the given format.
	 */
	fn.formatNumbers = function () {
		// TODO Implement this method.
		return this;
	};
	

	/* -- FINAL METHODS -- */

	/**
	 * Returns an array separated by the separator value.
	 *
	 * Example: The separator " " converts "My cool example" to ["My", "cool", "example"]
	 *
	 * @returns {array}
	 *		The splitted content text as an array.
	 */
	fn.toArray = function () {
		// TODO Implement this method.
	};

	/**
	 * Returns the content text.
	 */
	fn.toString = function () {
		return this.text;
	};

	window.texter = function (content) {
		return new Texter(content);
	};
	window.texter.Texter = Texter;

}(this));
